/*Tutor Paolo Pasteris
  Progetto multimediale, Comunicazione digitale, 
  Università degli Studi di Milano 13/14
  Composizione di un'immagine layer con chiave cromatica e una di sfondo 
  Chroma Key usa lo spazio colore HSB
*/
import java.awt.Color;

PImage layer,backgr,img_B_W,img_alfa;

float targetHue;
float Htollerance,
      Stollerance,
      Vtollerance;

void setup() {
  // Processing supporta immagini formato: gif, jpg e png
  // si usa loadImage(id) per caricare una foto esterna e 
  // createImage (w, h, format) per generarne una 
  backgr = loadImage("laboratorio_analisi.jpg");
  layer = loadImage("scienzata.jpg");
  img_B_W=createImage(layer.width,layer.height,RGB);
  img_alfa=createImage(layer.width,layer.height,ARGB);
  //valori presi con gimp e normalizzati tra 0 e 1
  targetHue=134f/360f;
  //targetHue=45f/360f;
  Htollerance=0.15f;
  Stollerance=0.3f;
  Vtollerance=0.3f;
  
  // la mascherina per la visualizzazione delle 3 img computate
  size(backgr.width*3,backgr.height);
  noLoop();
  background(255);
}

void draw() {
  // elaborazione della immagine layer e combinazione con l'immagine di sfondo
  chromaKey(layer,backgr);
  // visualizza l'immagine ricreata in prima posizione
  image(backgr,0, 0);
  // visualizza immagine B/N -bianco il layer, nero lo sfondo- in seconda pos
  image(img_B_W, img_B_W.width, 0);
  // visualizza l'immagine layer con lo sfondo diventato canale alfa in terza pos
  image(img_alfa, img_alfa.width*2, 0);
  // e salvata come file in formato PNG
  savePNG(img_alfa);
}

void chromaKey(PImage layer,PImage backgr){
  int dimension=layer.width*layer.height;
  int r,g,b;
  float[] hsb=new float[3];
  float deltaHue,alpha_channel;
    
  backgr.loadPixels();
  layer.loadPixels();
  img_B_W.loadPixels();
  img_alfa.loadPixels();
  
  for(int i=0;i<dimension;i++){
    //per ogni pixel dell'immagine layer conversione dallo spazio colore RGB a HSB
    Color.RGBtoHSB((int)red(layer.pixels[i]),(int)green(layer.pixels[i]),(int)blue(layer.pixels[i]), hsb);
    //differenza di tonalità tra la tonalità cercata e quella corrente
    deltaHue = Math.abs(hsb[0]-targetHue);
    if(deltaHue < Htollerance && hsb[1]> Stollerance && hsb[2]> Vtollerance){
      alpha_channel=0;   //è un pixel del fondale in chiave cromatica   
    }else{
      alpha_channel=1;   //è un pixel dell'immagine layer da tenere
    }
    // costruisco il valore rgb del pixel del layer o dello sfondo
    r=(int)(alpha_channel*red(layer.pixels[i])+(1-alpha_channel)*red(backgr.pixels[i]));
    g=(int)(alpha_channel*green(layer.pixels[i])+(1-alpha_channel)*green(backgr.pixels[i]));
    b=(int)(alpha_channel*blue(layer.pixels[i])+(1-alpha_channel)*blue(backgr.pixels[i]));
    
    // immagine pos I
    // sovrappongo l'immagine layer allo sfondo; ottengo l'immagine obiettivo
    backgr.pixels[i]=color(r,g,b);
    
    // immagine pos II
    // immagine bianco/nero, nero lo sfondo, bianco il layer
    img_B_W.pixels[i] = color(alpha_channel*255,alpha_channel*255,alpha_channel*255);
    
    // immagine pos III 
    // immagine ricostruita con l'aggiunta del canale alfa
    img_alfa.pixels[i]=color(r,g,b,alpha_channel*255);
  }
  backgr.updatePixels();
  img_B_W.updatePixels();
  img_alfa.updatePixels();
}
void savePNG(PImage img){
  img.save("frame.png");
}
